@@ -128,15 +128,6 @@ class datablock : public std::list<category>
 	 */
 	bool is_valid() const;
 
-	/**
-	 * @brief Validates the content of this datablock and all its content
-	 * and updates or removes the audit_conform category to match the result.
-	 * 
-	 * @return true If the content is valid
-	 * @return false If the content is not valid
-	 */
-	bool is_valid();
-
 	/**
 	 * @brief Validates all contained data for valid links between parents and children
 	 * as defined in the validator
@@ -146,6 +137,14 @@ class datablock : public std::list<category>
 	 */
 	bool validate_links() const;
 
+	/**
+	 * @brief Strip removes all categories and items that are invalid according
+	 * to the assigned validator. Will also add a valid audit_conform block.
+	 * 
+	 * @return true if the remaining datablock is valid
+	 */
+	bool strip();
+
 	// --------------------------------------------------------------------
 
 	/**
@@ -184,6 +183,15 @@ class datablock : public std::list<category>
 	 */
 	const category *get(std::string_view name) const;
 
+
+	/**
+	 * @brief Return true if this datablock contains a non-empty category
+	 */
+	bool contains(std::string_view name) const
+	{
+		return get(name) != nullptr;
+	}
+
 	/**
 	 * @brief Tries to find a category with name @a name and will create a
 	 * new one if it is not found. The result is a tuple of an iterator
