@@ -106,8 +106,6 @@ class atom
 
 		atom_impl(const atom_impl &i) = default;
 
-		void prefetch();
-
 		int compare(const atom_impl &b) const;
 
 		// bool getAnisoU(float anisou[6]) const;
@@ -345,7 +343,7 @@ class atom
 	std::string get_auth_asym_id() const { return get_property("auth_asym_id"); }      ///< Return the auth_asym_id property
 	std::string get_auth_seq_id() const { return get_property("auth_seq_id"); }        ///< Return the auth_seq_id property
 	std::string get_auth_atom_id() const { return get_property("auth_atom_id"); }      ///< Return the auth_atom_id property
-	std::string get_auth_alt_id() const { return get_property("auth_alt_id"); }        ///< Return the auth_alt_id property
+	std::string get_auth_alt_id() const { return get_property("pdbx_auth_alt_id"); }   ///< Return the auth_alt_id property
 	std::string get_auth_comp_id() const { return get_property("auth_comp_id"); }      ///< Return the auth_comp_id property
 	std::string get_pdb_ins_code() const { return get_property("pdbx_PDB_ins_code"); } ///< Return the pdb_ins_code property
 
@@ -481,8 +479,8 @@ class residue
 		, m_compound_id(compoundID)
 		, m_asym_id(asymID)
 		, m_seq_id(seqID)
-		, m_auth_asym_id(authAsymID)
-		, m_auth_seq_id(authSeqID)
+		, m_pdb_strand_id(authAsymID)
+		, m_pdb_seq_num(authSeqID)
 		, m_pdb_ins_code(pdbInsCode)
 	{
 	}
@@ -509,9 +507,9 @@ class residue
 	const std::string &get_asym_id() const { return m_asym_id; } ///< Return the asym_id
 	int get_seq_id() const { return m_seq_id; }                  ///< Return the seq_id
 
-	const std::string get_auth_asym_id() const { return m_auth_asym_id; } ///< Return the auth_asym_id
-	const std::string get_auth_seq_id() const { return m_auth_seq_id; }   ///< Return the auth_seq_id
-	std::string get_pdb_ins_code() const { return m_pdb_ins_code; }       ///< Return the pdb_ins_code
+	const std::string get_pdb_strand_id() const { return m_pdb_strand_id; } ///< Return the pdb_strand_id
+	const std::string get_pdb_seq_num() const { return m_pdb_seq_num; }     ///< Return the pdb_seq_num
+	std::string get_pdb_ins_code() const { return m_pdb_ins_code; }         ///< Return the pdb_ins_code
 
 	const std::string &get_compound_id() const { return m_compound_id; } ///< Return the compound_id
 	void set_compound_id(const std::string &id) { m_compound_id = id; }  ///< Set the compound_id to @a id
@@ -580,7 +578,7 @@ class residue
 								   m_seq_id == rhs.m_seq_id and
 								   m_asym_id == rhs.m_asym_id and
 								   m_compound_id == rhs.m_compound_id and
-								   m_auth_seq_id == rhs.m_auth_seq_id);
+								   m_pdb_seq_num == rhs.m_pdb_seq_num);
 	}
 
 	/// @brief Create a new atom and add it to the list
@@ -594,7 +592,7 @@ class residue
 	structure *m_structure = nullptr;
 	std::string m_compound_id, m_asym_id;
 	int m_seq_id = 0;
-	std::string m_auth_asym_id, m_auth_seq_id, m_pdb_ins_code;
+	std::string m_pdb_strand_id, m_pdb_seq_num, m_pdb_ins_code;
 	std::vector<atom> m_atoms;
 	/** @endcond */
 };
@@ -714,15 +712,15 @@ class polymer : public std::vector<monomer>
 
 	structure *get_structure() const { return m_structure; } ///< Return the structure
 
-	std::string get_asym_id() const { return m_asym_id; }           ///< Return the asym_id
-	std::string get_auth_asym_id() const { return m_auth_asym_id; } ///< Return the PDB chain ID, actually
-	std::string get_entity_id() const { return m_entity_id; }       ///< Return the entity_id
+	std::string get_asym_id() const { return m_asym_id; }             ///< Return the asym_id
+	std::string get_pdb_strand_id() const { return m_pdb_strand_id; } ///< Return the PDB chain ID, actually
+	std::string get_entity_id() const { return m_entity_id; }         ///< Return the entity_id
 
   private:
 	structure *m_structure;
 	std::string m_entity_id;
 	std::string m_asym_id;
-	std::string m_auth_asym_id;
+	std::string m_pdb_strand_id;
 };
 
 // --------------------------------------------------------------------
@@ -760,7 +758,7 @@ class sugar : public residue
 	int num() const
 	{
 		int result;
-		auto r = std::from_chars(m_auth_seq_id.data(), m_auth_seq_id.data() + m_auth_seq_id.length(), result);
+		auto r = std::from_chars(m_pdb_seq_num.data(), m_pdb_seq_num.data() + m_pdb_seq_num.length(), result);
 		if ((bool)r.ec)
 			throw std::runtime_error("The auth_seq_id should be a number for a sugar");
 		return result;
@@ -859,19 +857,38 @@ class branch : public std::vector<sugar>
 	std::string m_asym_id, m_entity_id;
 };
 
-// --------------------------------------------------------------------
-
-/// \brief A still very limited set of options for reading structures
-enum class StructureOpenOptions
+/** @brief Enumeration for controlling atom selection based on occupancy. */
+enum class occupancy_policy
 {
-	SkipHydrogen = 1 << 0 ///< Do not include hydrogen atoms in the structure object
+	/** @brief Include all atoms regardless of their occupancy factor. */
+	ALL = 0,
+
+	/** @brief Select only alternate atoms with the maximum occupancy factor.
+	 * If multiple atoms have the same maximum occupancy, choose the one with the minimum B-factor.
+	 * If multiple atoms share both the maximum occupancy and the minimum B-factor, select the first encountered atom.
+	 */
+	MAX = 1,
+
+	/** @brief Select only alternate atoms with the minimum occupancy factor.
+	 * Similar to MAX, if multiple atoms have the same minimum occupancy, choose the one with the minimum B-factor.
+	 * If multiple atoms share both the minimum occupancy and the minimum B-factor, select the first encountered atom.
+	 */
+	MIN = 2,
+
+	/** @brief Exclude all atoms with an occupancy factor greater than zero. */
+	UNOCCUPIED = 3
 };
 
-/// \brief A way to combine two options. Not very useful as there is only one...
-constexpr inline bool operator&(StructureOpenOptions a, StructureOpenOptions b)
+struct structure_open_options
 {
-	return static_cast<int>(a) bitand static_cast<int>(b);
-}
+	bool skip_hydrogen = false;                              ///< Do not include hydrogen atoms in the structure object
+	bool skip_hetatom = false;                               ///< Do not include HET atoms in the structure object
+	bool skip_water = false;                                 ///< Do not include water atoms in the structure object
+	occupancy_policy occupancy_mode = occupancy_policy::ALL; ///< By default, the occupancy policy is set to occupancy_policy::ALL
+	std::vector<std::string> asyms;                          ///< The asyms to load, if empty load all
+	std::optional<float> min_b_factor;                       ///< Only load atoms with at least this b_factor
+	std::optional<float> max_b_factor;                       ///< Only load atoms with at most this b_factor
+};
 
 // --------------------------------------------------------------------
 
@@ -885,10 +902,10 @@ class structure
 {
   public:
 	/// \brief Read the structure from cif::file @a p
-	structure(file &p, std::size_t modelNr = 1, StructureOpenOptions options = {});
+	structure(file &p, std::size_t modelNr = 1, structure_open_options options = {});
 
 	/// \brief Load the structure from already parsed mmCIF data in @a db
-	structure(datablock &db, std::size_t modelNr = 1, StructureOpenOptions options = {});
+	structure(datablock &db, std::size_t modelNr = 1, structure_open_options options = {});
 
 	/** @cond */
 	structure(structure &&s) = default;
@@ -1118,7 +1135,7 @@ class structure
 	friend polymer;
 	friend residue;
 
-	void load_atoms_for_model(StructureOpenOptions options);
+	void load_atoms_for_model(structure_open_options options);
 
 	std::string insert_compound(const std::string &compoundID, bool is_entity);
 
