@@ -26,26 +26,7 @@
 
 #pragma once
 
-#include "cif++/datablock.hpp"
+/// \file tls.hpp
+/// \deprecated This code has been moved to libpdb-redo
 
-#include <string>
-#include <tuple>
-#include <vector>
-
-namespace cif
-{
-
-struct tls_selection;
-struct tls_residue;
-
-struct tls_selection
-{
-	virtual ~tls_selection() {}
-	virtual void collect_residues(cif::datablock &db, std::vector<tls_residue> &residues, std::size_t indentLevel = 0) const = 0;
-	std::vector<std::tuple<std::string, int, int>> get_ranges(cif::datablock &db, bool pdbNamespace) const;
-};
-
-// Low level: get the selections
-std::unique_ptr<tls_selection> parse_tls_selection_details(const std::string &program, const std::string &selection);
-
-} // namespace cif
+#warning "This code has been moved to libpdb-redo"
