@@ -47,15 +47,10 @@ void atom::atom_impl::moveTo(const point &p)
 
 	auto r = row();
 
-#if __cpp_lib_format
-	r.assign("Cartn_x", std::format("{:.3f}", p.m_x), false, false);
-	r.assign("Cartn_y", std::format("{:.3f}", p.m_y), false, false);
-	r.assign("Cartn_z", std::format("{:.3f}", p.m_z), false, false);
-#else
-	r.assign("Cartn_x", cif::format("%.3f", p.m_x).str(), false, false);
-	r.assign("Cartn_y", cif::format("%.3f", p.m_y).str(), false, false);
-	r.assign("Cartn_z", cif::format("%.3f", p.m_z).str(), false, false);
-#endif
+	r.assign("Cartn_x", cif::format("{:.3f}", p.m_x), false, false);
+	r.assign("Cartn_y", cif::format("{:.3f}", p.m_y), false, false);
+	r.assign("Cartn_z", cif::format("{:.3f}", p.m_z), false, false);
+
 	m_location = p;
 }
 
@@ -102,18 +97,18 @@ void atom::atom_impl::set_property(const std::string_view name, const std::strin
 	r.assign(name, value, true, true);
 }
 
-// int atom::atom_impl::compare(const atom_impl &b) const
-// {
-// 	int d = m_asym_id.compare(b.m_asym_id);
-// 	if (d == 0)
-// 		d = m_seq_id - b.m_seq_id;
-// 	if (d == 0)
-// 		d = m_auth_seq_id.compare(b.m_auth_seq_id);
-// 	if (d == 0)
-// 		d = mAtom_id.compare(b.mAtom_id);
+int atom::atom_impl::compare(const atom_impl &b) const
+{
+	int d = get_property("label_asym_id").compare(b.get_property("label_asym_id"));
+	if (d == 0)
+		d = get_property_int("label_seq_id") - b.get_property_int("label_seq_id");
+	if (d == 0)
+		d = get_property_int("auth_seq_id") - b.get_property_int("auth_seq_id");
+	if (d == 0)
+		d = get_property("label_atom_id").compare(b.get_property("label_atom_id"));
 
-// 	return d;
-// }
+	return d;
+}
 
 // bool atom::atom_impl::getAnisoU(float anisou[6]) const
 // {
@@ -149,145 +144,6 @@ int atom::atom_impl::get_charge() const
 	return formalCharge.value_or(0);
 }
 
-// const Compound *atom::atom_impl::compound() const
-// {
-// 	if (mCompound == nullptr)
-// 	{
-// 		std::string compID = get_property("label_comp_id");
-
-// 		mCompound = compound_factory::instance().create(compID);
-// 	}
-
-// 	return mCompound;
-// }
-
-// const std::string atom::atom_impl::get_property(const std::string_view name) const
-// {
-// 	for (auto &&[item_name, ref] : mCachedRefs)
-// 	{
-// 		if (item_name == name)
-// 			return ref.as<std::string>();
-// 	}
-
-// 	mCachedRefs.emplace_back(name, const_cast<Row &>(mRow)[name]);
-// 	return std::get<1>(mCachedRefs.back()).as<std::string>();
-// }
-
-// void atom::atom_impl::set_property(const std::string_view name, const std::string &value)
-// {
-// 	for (auto &&[item_name, ref] : mCachedRefs)
-// 	{
-// 		if (item_name != name)
-// 			continue;
-
-// 		ref = value;
-// 		return;
-// 	}
-
-// 	mCachedRefs.emplace_back(name, mRow[name]);
-// 	std::get<1>(mCachedRefs.back()) = value;
-// }
-
-// const Row atom::getRowAniso() const
-// {
-// 	auto &db = m_impl->m_db;
-// 	auto cat = db.get("atom_site_anisotrop");
-// 	if (not cat)
-// 		return {};
-// 	else
-// 		return cat->find1(key("id") == m_impl->m_id);
-// }
-
-// float atom::uIso() const
-// {
-// 	float result;
-
-// 	if (not get_property<std::string>("U_iso_or_equiv").empty())
-// 		result = get_property<float>("U_iso_or_equiv");
-// 	else if (not get_property<std::string>("B_iso_or_equiv").empty())
-// 		result = get_property<float>("B_iso_or_equiv") / static_cast<float>(8 * kPI * kPI);
-// 	else
-// 		throw std::runtime_error("Missing B_iso or U_iso");
-
-// 	return result;
-// }
-
-// const Compound &atom::compound() const
-// {
-// 	auto result = impl().compound();
-
-// 	if (result == nullptr)
-// 	{
-// 		if (VERBOSE > 0)
-// 			std::cerr << "Compound not found: '" << get_property<std::string>("label_comp_id") << '\'' << '\n';
-
-// 		throw std::runtime_error("no compound");
-// 	}
-
-// 	return *result;
-// }
-
-// std::string atom::labelEntityID() const
-// {
-// 	return get_property<std::string>("label_entity_id");
-// }
-
-// std::string atom::authAtom_id() const
-// {
-// 	return get_property<std::string>("auth_atom_id");
-// }
-
-// std::string atom::authCompID() const
-// {
-// 	return get_property<std::string>("auth_comp_id");
-// }
-
-// std::string atom::get_auth_asym_id() const
-// {
-// 	return get_property<std::string>("auth_asym_id");
-// }
-
-// std::string atom::get_pdb_ins_code() const
-// {
-// 	return get_property<std::string>("pdbx_PDB_ins_code");
-// }
-
-// std::string atom::pdbxAuthAltID() const
-// {
-// 	return get_property<std::string>("pdbx_auth_alt_id");
-// }
-
-// void atom::translate(point t)
-// {
-// 	auto loc = location();
-// 	loc += t;
-// 	location(loc);
-// }
-
-// void atom::rotate(quaternion q)
-// {
-// 	auto loc = location();
-// 	loc.rotate(q);
-// 	location(loc);
-// }
-
-// void atom::translate_and_rotate(point t, quaternion q)
-// {
-// 	auto loc = location();
-// 	loc += t;
-// 	loc.rotate(q);
-// 	location(loc);
-// }
-
-// void atom::translate_rotate_and_translate(point t1, quaternion q, point t2)
-// {
-// 	auto loc = location();
-// 	loc += t1;
-// 	loc.rotate(q);
-// 	loc += t2;
-// 	location(loc);
-// }
-
 std::ostream &operator<<(std::ostream &os, const atom &atom)
 {
 	if (atom.is_water())
@@ -319,8 +175,8 @@ residue::residue(structure &structure, const std::vector<atom> &atoms)
 	m_compound_id = a.get_label_comp_id();
 	m_asym_id = a.get_label_asym_id();
 	m_seq_id = a.get_label_seq_id();
-	m_auth_asym_id = a.get_auth_asym_id();
-	m_auth_seq_id = a.get_auth_seq_id();
+	m_pdb_strand_id = a.get_auth_asym_id();
+	m_pdb_seq_num = a.get_auth_seq_id();
 	m_pdb_ins_code = a.get_pdb_ins_code();
 
 	for (auto atom : atoms)
@@ -371,10 +227,10 @@ atom residue::create_new_atom(atom_type inType, const std::string &inAtomID, poi
 		{ "label_alt_id", "." },
 		{ "label_comp_id", m_compound_id },
 		{ "label_seq_id", m_seq_id },
-		{ "auth_asym_id", m_auth_asym_id },
+		{ "auth_asym_id", m_pdb_strand_id },
 		{ "auth_atom_id", inAtomID },
 		{ "auth_comp_id", m_compound_id },
-		{ "auth_seq_id", m_auth_seq_id },
+		{ "auth_seq_id", m_pdb_seq_num },
 		{ "occupancy", 1.0f, 2 },
 		{ "B_iso_or_equiv", 20.0f },
 		{ "pdbx_PDB_model_num", m_structure->get_model_nr() },
@@ -541,8 +397,8 @@ std::ostream &operator<<(std::ostream &os, const residue &res)
 {
 	os << res.get_compound_id() << ' ' << res.get_asym_id() << ':' << res.get_seq_id();
 
-	if (res.get_auth_asym_id() != res.get_asym_id() or res.get_auth_seq_id() != std::to_string(res.get_seq_id()))
-		os << " [" << res.get_auth_asym_id() << ':' << res.get_auth_seq_id() << ']';
+	if (res.get_pdb_strand_id() != res.get_asym_id() or res.get_pdb_seq_num() != std::to_string(res.get_seq_id()))
+		os << " [" << res.get_pdb_strand_id() << ':' << res.get_pdb_seq_num() << ']';
 
 	return os;
 }
@@ -551,7 +407,7 @@ std::ostream &operator<<(std::ostream &os, const residue &res)
 // monomer
 
 monomer::monomer(const polymer &polymer, std::size_t index, int seqID, const std::string &authSeqID, const std::string &pdbInsCode, const std::string &compoundID)
-	: residue(*polymer.get_structure(), compoundID, polymer.get_asym_id(), seqID, polymer.get_auth_asym_id(), authSeqID, pdbInsCode)
+	: residue(*polymer.get_structure(), compoundID, polymer.get_asym_id(), seqID, polymer.get_pdb_strand_id(), authSeqID, pdbInsCode)
 	, m_polymer(&polymer)
 	, m_index(index)
 {
@@ -970,7 +826,7 @@ polymer::polymer(structure &s, const std::string &entityID, const std::string &a
 	: m_structure(const_cast<structure *>(&s))
 	, m_entity_id(entityID)
 	, m_asym_id(asym_id)
-	, m_auth_asym_id(auth_asym_id)
+	, m_pdb_strand_id(auth_asym_id)
 {
 	using namespace cif::literals;
 
@@ -982,12 +838,8 @@ polymer::polymer(structure &s, const std::string &entityID, const std::string &a
 	for (auto r : poly_seq_scheme.find("asym_id"_key == asym_id))
 	{
 		int seqID;
-		std::optional<int> pdbSeqNum;
-		std::string compoundID, authSeqID, pdbInsCode;
-		cif::tie(seqID, authSeqID, compoundID, pdbInsCode, pdbSeqNum) = r.get("seq_id", "auth_seq_num", "mon_id", "pdb_ins_code", "pdb_seq_num");
-
-		if (authSeqID.empty() and pdbSeqNum.has_value())
-			authSeqID = std::to_string(*pdbSeqNum);
+		std::string compoundID, pdbSeqNum, pdbInsCode;
+		cif::tie(seqID, pdbSeqNum, compoundID, pdbInsCode) = r.get("seq_id", "pdb_seq_num", "mon_id", "pdb_ins_code");
 
 		std::size_t index = size();
 
@@ -995,11 +847,11 @@ polymer::polymer(structure &s, const std::string &entityID, const std::string &a
 		if (not ix.count(seqID))
 		{
 			ix[seqID] = index;
-			emplace_back(*this, index, seqID, authSeqID, pdbInsCode, compoundID);
+			emplace_back(*this, index, seqID, pdbSeqNum, pdbInsCode, compoundID);
 		}
 		else if (VERBOSE > 0)
 		{
-			monomer m{ *this, index, seqID, authSeqID, pdbInsCode, compoundID };
+			monomer m{ *this, index, seqID, pdbSeqNum, pdbInsCode, compoundID };
 			std::cerr << "Dropping alternate residue " << m << '\n';
 		}
 	}
@@ -1139,7 +991,7 @@ cif::mm::atom sugar::add_atom(row_initializer atom_info)
 	atom_info.set_value({ "label_alt_id", "." });
 	atom_info.set_value({ "auth_asym_id", m_branch->get_asym_id() });
 	atom_info.set_value({ "auth_comp_id", m_compound_id });
-	atom_info.set_value({ "auth_seq_id", m_auth_seq_id });
+	atom_info.set_value({ "auth_seq_id", m_pdb_seq_num });
 	atom_info.set_value({ "occupancy", 1.0, 2 });
 	atom_info.set_value({ "B_iso_or_equiv", 30.0, 2 });
 	atom_info.set_value({ "pdbx_PDB_model_num", 1 });
@@ -1255,12 +1107,12 @@ sugar &branch::construct_sugar(const std::string &compound_id)
 		{ "mon_id", result.get_compound_id() },
 
 		{ "pdb_asym_id", result.get_asym_id() },
-		{ "pdb_seq_num", result.num() },
+		{ "pdb_seq_num", result.get_pdb_seq_num() },
 		{ "pdb_mon_id", result.get_compound_id() },
 
-		{ "auth_asym_id", result.get_auth_asym_id() },
+		{ "auth_asym_id", result.get_pdb_strand_id() },
 		{ "auth_mon_id", result.get_compound_id() },
-		{ "auth_seq_num", result.get_auth_seq_id() },
+		{ "auth_seq_num", result.get_pdb_seq_num() },
 
 		{ "hetero", "n" } });
 
@@ -1303,7 +1155,7 @@ std::string branch::name(const sugar &s) const
 
 	for (auto &sn : *this)
 	{
-		if (not sn.get_link() or sn.get_link().get_auth_seq_id() != s.get_auth_seq_id())
+		if (not sn.get_link() or sn.get_link().get_auth_seq_id() != s.get_pdb_seq_num())
 			continue;
 
 		auto n = name(sn) + "-(1-" + sn.get_link().get_label_atom_id().substr(1) + ')';
@@ -1330,19 +1182,19 @@ float branch::weight() const
 // --------------------------------------------------------------------
 //	structure
 
-structure::structure(file &p, std::size_t modelNr, StructureOpenOptions options)
+structure::structure(file &p, std::size_t modelNr, structure_open_options options)
 	: structure(p.front(), modelNr, options)
 {
 }
 
-structure::structure(datablock &db, std::size_t modelNr, StructureOpenOptions options)
+structure::structure(datablock &db, std::size_t modelNr, structure_open_options options)
 	: m_db(db)
 	, m_model_nr(modelNr)
 {
 	if (db.get_validator() == nullptr)
 		db.load_dictionary();
 
-	auto &atomCat = db["atom_site"];
+	auto &atom_site = db["atom_site"];
 
 	load_atoms_for_model(options);
 
@@ -1350,7 +1202,7 @@ structure::structure(datablock &db, std::size_t modelNr, StructureOpenOptions op
 	if (m_atoms.empty() and m_model_nr == 1)
 	{
 		std::optional<std::size_t> model_nr;
-		cif::tie(model_nr) = atomCat.front().get("pdbx_PDB_model_num");
+		cif::tie(model_nr) = atom_site.front().get("pdbx_PDB_model_num");
 		if (model_nr and *model_nr != m_model_nr)
 		{
 			if (VERBOSE > 0)
@@ -1369,42 +1221,133 @@ structure::structure(datablock &db, std::size_t modelNr, StructureOpenOptions op
 		load_data();
 }
 
-void structure::load_atoms_for_model(StructureOpenOptions options)
+void structure::load_atoms_for_model(structure_open_options options)
 {
 	using namespace literals;
 
-	auto &atomCat = m_db["atom_site"];
+	auto &atom_site = m_db["atom_site"];
 
 	condition c = "pdbx_PDB_model_num"_key == null or "pdbx_PDB_model_num"_key == m_model_nr;
-	if (options bitand StructureOpenOptions::SkipHydrogen)
-		c = std::move(c) and ("type_symbol"_key != "H" and "type_symbol"_key != "D");
 
-	for (auto id : atomCat.find<std::string>(std::move(c), "id"))
-		emplace_atom(std::make_shared<atom::atom_impl>(m_db, id));
+	if (options.skip_hydrogen)
+		c = std::move(c) and (cif::key("type_symbol") != "H" and cif::key("type_symbol") != "D");
+
+	if (options.skip_water)
+		c = std::move(c) and (cif::key("auth_comp_id") != "HOH" and cif::key("auth_comp_id") != "H20" and cif::key("auth_comp_id") != "WAT");
+
+	if (options.skip_hetatom)
+	{
+		if (options.skip_water)
+			c = std::move(c) and cif::key("group_PDB") != "HETATM";
+		else
+			c = std::move(c) and (cif::key("group_PDB") != "HETATM" or (cif::key("auth_comp_id") == "HOH" or cif::key("auth_comp_id") == "H20" or cif::key("auth_comp_id") == "WAT"));
+	}
+
+	if (options.min_b_factor.has_value())
+		c = std::move(c) and cif::key("B_iso_or_equiv") >= *options.min_b_factor;
+
+	if (options.max_b_factor.has_value())
+		c = std::move(c) and cif::key("B_iso_or_equiv") <= *options.max_b_factor;
+
+	if (not options.asyms.empty())
+	{
+		condition tmp_c;
+		for (auto asym_id : options.asyms)
+			tmp_c = std::move(tmp_c) or cif::key("label_asym_id") == asym_id;
+		c = std::move(c) and std::move(tmp_c);
+	}
+
+	if (options.occupancy_mode == occupancy_policy::ALL)
+	{
+		for (auto id : atom_site.find<std::string>(std::move(c), "id"))
+			emplace_atom(std::make_shared<atom::atom_impl>(m_db, id));
+	}
+	else if (options.occupancy_mode == occupancy_policy::UNOCCUPIED)
+	{
+		for (auto id : atom_site.find<std::string>(std::move(c), "id"))
+		{
+			auto a = std::make_shared<atom::atom_impl>(m_db, id);
+			if (a->get_property_float("occupancy") > 0)
+				continue;
+			emplace_atom(a);
+		}
+	}
+	else
+	{
+		std::vector<cif::mm::atom> atoms;
+		std::map<std::tuple<std::string,int>, std::map<std::string, float>> alts;
+	
+		for (auto id : atom_site.find<std::string>(std::move(c), "id"))
+		{
+			auto a = atoms.emplace_back(std::make_shared<atom::atom_impl>(m_db, id));
+	
+			if (a.is_alternate())
+			{
+				auto key = std::make_tuple(a.get_label_asym_id(), a.get_label_seq_id());
+				auto alt_id = a.get_label_alt_id();
+	
+				if (auto i = alts.find(key); i != alts.end())
+					i->second[alt_id] += a.get_occupancy();
+				else
+					alts[key][alt_id] = a.get_occupancy();
+			}
+		}
+	
+		for (auto &&[key, value] : alts)
+		{
+			// const auto &[asym_id, seq_id] = key;
+	
+			// select highest occupancy for this residue's alternates
+			std::string alt_id;
+			float occupancy = options.occupancy_mode == occupancy_policy::MAX ? 0.f : std::numeric_limits<float>::max();
+			for (const auto &[alt_key, alt_value] : value)
+			{
+				if (options.occupancy_mode == occupancy_policy::MAX)
+				{
+					if (occupancy < alt_value)
+					{
+						alt_id = alt_key;
+						occupancy = alt_value;
+					}
+				}
+				else
+				{
+					if (occupancy > alt_value)
+					{
+						alt_id = alt_key;
+						occupancy = alt_value;
+					}
+				}
+			}
+	
+			value.clear();
+			value.emplace(alt_id, occupancy);
+		}
+	
+		for (auto a : atoms)
+		{
+			if (a.is_alternate())
+			{
+				auto key = std::make_tuple(a.get_label_asym_id(), a.get_label_seq_id());
+
+				if (alts[key].contains(a.get_label_alt_id()))
+					emplace_atom(a);
+			}
+			else
+				emplace_atom(a);
+	
+		}
+	}
+
 }
 
-// structure::structure(const structure &s)
-// 	: m_db(s.m_db)
-// 	, m_model_nr(s.m_model_nr)
-// {
-// 	m_atoms.reserve(s.m_atoms.size());
-// 	for (auto &atom : s.m_atoms)
-// 		emplace_atom(atom.clone());
-
-// 	load_data();
-// }
-
-// structure::~structure()
-// {
-// }
-
 void structure::load_data()
 {
 	auto &polySeqScheme = m_db["pdbx_poly_seq_scheme"];
 
 	for (const auto &[asym_id, auth_asym_id, entityID] : polySeqScheme.rows<std::string, std::string, std::string>("asym_id", "pdb_strand_id", "entity_id"))
 	{
-		if (m_polymers.empty() or m_polymers.back().get_asym_id() != asym_id or m_polymers.back().get_entity_id() != entityID)
+		if (m_polymers.empty() or m_polymers.back().get_asym_id() != asym_id)
 			m_polymers.emplace_back(*this, entityID, asym_id, auth_asym_id);
 	}
 
@@ -1430,18 +1373,18 @@ void structure::load_data()
 	for (auto &poly : m_polymers)
 	{
 		for (auto &res : poly)
-			resMap[{ res.get_asym_id(), res.get_seq_id(), res.get_auth_seq_id() }] = &res;
+			resMap[{ res.get_asym_id(), res.get_seq_id(), res.get_pdb_seq_num() }] = &res;
 	}
 
 	for (auto &res : m_non_polymers)
-		resMap[{ res.get_asym_id(), res.get_seq_id(), res.get_auth_seq_id() }] = &res;
+		resMap[{ res.get_asym_id(), res.get_seq_id(), res.get_pdb_seq_num() }] = &res;
 
 	std::set<std::string> sugars;
 	for (auto &branch : m_branches)
 	{
 		for (auto &sugar : branch)
 		{
-			resMap[{ sugar.get_asym_id(), sugar.get_seq_id(), sugar.get_auth_seq_id() }] = &sugar;
+			resMap[{ sugar.get_asym_id(), sugar.get_seq_id(), sugar.get_pdb_seq_num() }] = &sugar;
 			sugars.insert(sugar.get_compound_id());
 		}
 	}
@@ -1516,30 +1459,6 @@ EntityType structure::get_entity_type_for_asym_id(const std::string asym_id) con
 	return get_entity_type_for_entity_id(entityID);
 }
 
-// std::vector<atom> structure::waters() const
-// {
-// 	using namespace literals;
-
-// 	std::vector<atom> result;
-
-// 	auto &db = datablock();
-
-// 	// Get the entity id for water. Watch out, structure may not have water at all
-// 	auto &entityCat = db["entity"];
-// 	for (const auto &[waterEntityID] : entityCat.find<std::string>("type"_key == "water", "id"))
-// 	{
-// 		for (auto &a : m_atoms)
-// 		{
-// 			if (a.get_property("label_entity_id") == waterEntityID)
-// 				result.push_back(a);
-// 		}
-
-// 		break;
-// 	}
-
-// 	return result;
-// }
-
 bool structure::has_atom_id(const std::string &id) const
 {
 	assert(m_atoms.size() == m_atom_index.size());
@@ -1688,7 +1607,7 @@ residue &structure::get_residue(const std::string &asym_id, int seqID, const std
 	{
 		for (auto &res : m_non_polymers)
 		{
-			if (res.get_asym_id() == asym_id and (authSeqID.empty() or res.get_auth_seq_id() == authSeqID))
+			if (res.get_asym_id() == asym_id and (authSeqID.empty() or res.get_pdb_seq_num() == authSeqID))
 				return res;
 		}
 	}
@@ -1712,7 +1631,7 @@ residue &structure::get_residue(const std::string &asym_id, int seqID, const std
 
 		for (auto &sugar : branch)
 		{
-			if (sugar.get_asym_id() == asym_id and sugar.get_auth_seq_id() == authSeqID)
+			if (sugar.get_asym_id() == asym_id and sugar.get_pdb_seq_num() == authSeqID)
 				return sugar;
 		}
 	}
@@ -1734,7 +1653,7 @@ residue &structure::get_residue(const std::string &asym_id, const std::string &c
 	{
 		for (auto &res : m_non_polymers)
 		{
-			if (res.get_asym_id() == asym_id and res.get_auth_seq_id() == authSeqID and res.get_compound_id() == compID)
+			if (res.get_asym_id() == asym_id and res.get_pdb_seq_num() == authSeqID and res.get_compound_id() == compID)
 				return res;
 		}
 	}
@@ -1758,7 +1677,7 @@ residue &structure::get_residue(const std::string &asym_id, const std::string &c
 
 		for (auto &sugar : branch)
 		{
-			if (sugar.get_asym_id() == asym_id and sugar.get_auth_seq_id() == authSeqID and sugar.get_compound_id() == compID)
+			if (sugar.get_asym_id() == asym_id and sugar.get_pdb_seq_num() == authSeqID and sugar.get_compound_id() == compID)
 				return sugar;
 		}
 	}
@@ -2108,7 +2027,7 @@ void structure::remove_residue(const std::string &asym_id, int seq_id, const std
 	{
 		for (auto &res : m_non_polymers)
 		{
-			if (res.get_asym_id() == asym_id and (auth_seq_id.empty() or res.get_auth_seq_id() == auth_seq_id))
+			if (res.get_asym_id() == asym_id and (auth_seq_id.empty() or res.get_pdb_seq_num() == auth_seq_id))
 			{
 				remove_residue(res);
 				return;
@@ -2138,7 +2057,7 @@ void structure::remove_residue(const std::string &asym_id, int seq_id, const std
 
 		for (auto &sugar : branch)
 		{
-			if (sugar.get_asym_id() == asym_id and sugar.get_auth_seq_id() == auth_seq_id)
+			if (sugar.get_asym_id() == asym_id and sugar.get_pdb_seq_num() == auth_seq_id)
 			{
 				remove_residue(sugar);
 				return;
@@ -2271,7 +2190,7 @@ void structure::remove_sugar(sugar &s)
 				// TODO: need fix, collect from nag_atoms?
 				{ "auth_asym_id", asym_id },
 				{ "auth_mon_id", sugar.get_compound_id() },
-				{ "auth_seq_num", sugar.get_auth_seq_id() },
+				{ "auth_seq_num", sugar.get_pdb_seq_num() },
 
 				{ "hetero", "n" } });
 		}
@@ -2357,8 +2276,8 @@ std::string structure::create_non_poly(const std::string &entity_id, const std::
 		{ "entity_id", entity_id },
 		{ "mon_id", comp_id },
 		{ "ndb_seq_num", ndb_nr },
-		{ "pdb_seq_num", res.get_auth_seq_id() },
-		{ "auth_seq_num", res.get_auth_seq_id() },
+		{ "pdb_seq_num", res.get_pdb_seq_num() },
+		{ "auth_seq_num", res.get_pdb_seq_num() },
 		{ "pdb_mon_id", comp_id },
 		{ "auth_mon_id", comp_id },
 		{ "pdb_strand_id", asym_id },
@@ -2418,8 +2337,8 @@ std::string structure::create_non_poly(const std::string &entity_id, std::vector
 		{ "entity_id", entity_id },
 		{ "mon_id", comp_id },
 		{ "ndb_seq_num", ndb_nr },
-		{ "pdb_seq_num", res.get_auth_seq_id() },
-		{ "auth_seq_num", res.get_auth_seq_id() },
+		{ "pdb_seq_num", res.get_pdb_seq_num() },
+		{ "auth_seq_num", res.get_pdb_seq_num() },
 		{ "pdb_mon_id", comp_id },
 		{ "auth_mon_id", comp_id },
 		{ "pdb_strand_id", asym_id },
@@ -2733,11 +2652,11 @@ std::string structure::create_entity_for_branch(branch &branch)
 
 			pdbx_entity_branch_link.emplace({ { "link_id", pdbx_entity_branch_link.get_unique_id("") },
 				{ "entity_id", entityID },
-				{ "entity_branch_list_num_1", s1.get_auth_seq_id() },
+				{ "entity_branch_list_num_1", s1.get_pdb_seq_num() },
 				{ "comp_id_1", s1.get_compound_id() },
 				{ "atom_id_1", l1.get_label_atom_id() },
 				{ "leaving_atom_id_1", "O1" },
-				{ "entity_branch_list_num_2", s2.get_auth_seq_id() },
+				{ "entity_branch_list_num_2", s2.get_pdb_seq_num() },
 				{ "comp_id_2", s2.get_compound_id() },
 				{ "atom_id_2", l2.get_label_atom_id() },
 				{ "leaving_atom_id_2", "H" + l2.get_label_atom_id() },
@@ -2950,7 +2869,7 @@ static int compare_numbers(std::string_view a, std::string_view b)
 
 int compare_cif_id(const std::string &a, const std::string &b)
 {
-	int d = a.length() - b.length();
+	int d = static_cast<int>(a.length() - b.length());
 	if (d == 0)
 		d = a.compare(b);
 	return d;
