@@ -25,6 +25,7 @@
  */
 
 #include "cif++.hpp"
+#include "cif++/row.hpp"
 
 // --------------------------------------------------------------------
 
@@ -138,15 +139,15 @@ void checkEntities(datablock &db)
 				auto compound = cf.create(comp_id);
 				if (compound)
 					formula_weight += compound->formula_weight();
-				else if (cif::VERBOSE > 0)
-					std::clog << "missing information for compound " + comp_id << '\n';
+				// else if (cif::VERBOSE > 0)
+				// 	std::clog << "missing information for compound " + comp_id << '\n';
 				++n;
 			}
 
-			formula_weight -= (n - 1) * 18.015;
+			formula_weight -= (n - 1) * 18.015f;
 		}
 		else if (type == "water")
-			formula_weight = 18.015;
+			formula_weight = 18.015f;
 		else if (type == "branched")
 		{
 			int n = 0;
@@ -156,12 +157,12 @@ void checkEntities(datablock &db)
 				auto compound = cf.create(comp_id);
 				if (compound)
 					formula_weight += compound->formula_weight();
-				else if (cif::VERBOSE > 0)
-					std::clog << "missing information for compound " + comp_id << '\n';
+				// else if (cif::VERBOSE > 0)
+				// 	std::clog << "missing information for compound " + comp_id << '\n';
 				++n;
 			}
 
-			formula_weight -= (n - 1) * 18.015;
+			formula_weight -= (n - 1) * 18.015f;
 		}
 		else if (type == "non-polymer")
 		{
@@ -171,7 +172,7 @@ void checkEntities(datablock &db)
 				auto compound = cf.create(*comp_id);
 				if (not compound)
 				{
-					std::cerr << "missing information for compound " << *comp_id << "\n";
+					// std::cerr << "missing information for compound " << *comp_id << "\n";
 					continue;
 				}
 				formula_weight = compound->formula_weight();
@@ -264,8 +265,8 @@ void createEntityIDs(datablock &db)
 
 		std::string comp_id = get_comp_id(k);
 
-		for (auto &k : e)
-			atom_site.update_value(get_condition(k), "label_entity_id", entity_id);
+		for (auto &k2 : e)
+			atom_site.update_value(get_condition(k2), "label_entity_id", entity_id);
 	}
 
 	if (not waters.empty())
@@ -319,7 +320,7 @@ void fillLabelAsymID(category &atom_site)
 		{
 			if (not mapAuthAsymIDAndEntityToLabelAsymID.contains(key))
 			{
-				std::string asym_id = cif_id_for_number(mapAuthAsymIDAndEntityToLabelAsymID.size());
+				std::string asym_id = cif_id_for_number(static_cast<int>(mapAuthAsymIDAndEntityToLabelAsymID.size()));
 				mapAuthAsymIDAndEntityToLabelAsymID[key] = asym_id;
 			}
 		}
@@ -484,7 +485,8 @@ void checkAtomRecords(datablock &db)
 		if (not compound)
 		{
 			missingCompounds.insert(comp_id);
-			std::cerr << "Missing compound information for " << comp_id << "\n";
+			// if (cif::VERBOSE > 0)
+			// 	std::cerr << "Missing compound information for " << comp_id << "\n";
 			continue;
 		}
 
@@ -542,7 +544,7 @@ void checkAtomRecords(datablock &db)
 
 		// Rewrite the coordinates and other items that look better in a fixed format
 		// Be careful not to nuke invalidly formatted data here
-		for (auto [item_name, prec] : std::vector<std::tuple<std::string_view, std::string::size_type>>{
+		for (auto [item_name, prec] : std::vector<std::tuple<std::string_view, int>>{
 				 { "cartn_x", 3 },
 				 { "cartn_y", 3 },
 				 { "cartn_z", 3 },
@@ -557,11 +559,11 @@ void checkAtomRecords(datablock &db)
 			if (auto [ptr, ec] = cif::from_chars(s.data(), s.data() + s.length(), v); (bool)ec)
 				continue;
 
-			if (s.length() < prec + 1 or s[s.length() - prec - 1] != '.')
+			if (s.length() < prec + 1UL or s[s.length() - prec - 1] != '.')
 			{
 				char b[12];
 
-				if (auto [ptr, ec] = cif::to_chars(b, b + sizeof(b), v, cif::chars_format::fixed, prec); (bool)ec)
+				if (auto [ptr, ec] = cif::to_chars(b, b + sizeof(b), v, cif::chars_format::fixed, prec); ec == std::errc{})
 					row.assign(item_name, { b, static_cast<std::string::size_type>(ptr - b) }, false, false);
 			}
 		}
@@ -628,13 +630,13 @@ void checkAtomAnisotropRecords(datablock &db)
 
 		if (row["pdbx_auth_alt_id"].empty() and not parent["pdbx_auth_alt_id"].empty())
 			row["pdbx_auth_alt_id"] = parent["pdbx_auth_alt_id"].text();
-		if (row["pdbx_label_seq_id"].empty() and not parent["pdbx_label_seq_id"].empty())
+		if (row["pdbx_label_seq_id"].empty() and not parent["label_seq_id"].empty())
 			row["pdbx_label_seq_id"] = parent["label_seq_id"].text();
-		if (row["pdbx_label_asym_id"].empty() and not parent["pdbx_label_asym_id"].empty())
+		if (row["pdbx_label_asym_id"].empty() and not parent["label_asym_id"].empty())
 			row["pdbx_label_asym_id"] = parent["label_asym_id"].text();
-		if (row["pdbx_label_atom_id"].empty() and not parent["pdbx_label_atom_id"].empty())
+		if (row["pdbx_label_atom_id"].empty() and not parent["label_atom_id"].empty())
 			row["pdbx_label_atom_id"] = parent["label_atom_id"].text();
-		if (row["pdbx_label_comp_id"].empty() and not parent["pdbx_label_comp_id"].empty())
+		if (row["pdbx_label_comp_id"].empty() and not parent["label_comp_id"].empty())
 			row["pdbx_label_comp_id"] = parent["label_comp_id"].text();
 		// if (row["pdbx_PDB_model_num"].empty() and not parent["pdbx_PDB_model_num"].empty())
 		// 	row["pdbx_PDB_model_num"] = parent["pdbx_PDB_model_num"].text();
@@ -722,7 +724,7 @@ void createEntity(datablock &db)
 
 		std::string type, desc;
 		float weight = 0;
-		int count = 0;
+		size_t count = 0;
 
 		auto first_comp_id = std::get<0>(content.front());
 
@@ -1167,7 +1169,7 @@ void createPdbxNonpolyScheme(datablock &db)
 		for (int ndb_nr = 1; auto row : atom_site.find("label_entity_id"_key == entity_id and "label_comp_id"_key == comp_id))
 		{
 			// Skip existing records
-			auto linked = atom_site.get_linked(row, pdbx_nonpoly_scheme);
+			auto linked = atom_site.get_children(row, pdbx_nonpoly_scheme);
 			if (not linked.empty())
 				continue;
 
@@ -1242,6 +1244,101 @@ void createPdbxBranchScheme(datablock &db)
 	}
 }
 
+void reconstruct_index_for_category(const validator &validator, category &cat, datablock &db)
+{
+	auto cv = validator.get_validator_for_category(cat.name());
+
+	enum class State
+	{
+		Start,
+		MissingKeys,
+		DuplicateKeys
+	} state = State::Start;
+
+	for (;;)
+	{
+		// See if we can build an index
+		try
+		{
+			cat.set_validator(&validator, db);
+		}
+		catch (const missing_key_error &ex)
+		{
+			if (state == State::MissingKeys)
+			{
+				if (cif::VERBOSE > 0)
+					std::clog << "Repairing failed for category " << cat.name() << ", missing keys remain: " << ex.what() << '\n';
+
+				throw;
+			}
+
+			state = State::MissingKeys;
+
+			auto key = ex.get_key();
+
+			if (cif::VERBOSE > 1)
+				std::clog << "Need to add key " << key << " to category " << cat.name() << '\n';
+
+			for (auto row : cat)
+			{
+				auto ord = row.get<std::string>(key.c_str());
+				if (ord.empty())
+					row.assign({ //
+						{ key, cat.get_unique_value(key) } });
+			}
+
+			continue;
+		}
+		catch (const duplicate_key_error &ex)
+		{
+			if (state == State::DuplicateKeys)
+			{
+				if (cif::VERBOSE > 0)
+					std::clog << "Repairing failed for category " << cat.name() << ", duplicate keys remain: " << ex.what() << '\n';
+
+				throw;
+			}
+
+			state = State::DuplicateKeys;
+
+			if (cif::VERBOSE > 0)
+				std::clog << "Attempt to fix " << cat.name() << " failed: " << ex.what() << '\n';
+
+			// replace items that do not define a relation to a parent
+
+			std::set<std::string> replaceableKeys;
+			for (auto key : cv->m_keys)
+			{
+				bool replaceable = true;
+				for (auto lv : validator.get_links_for_child(cat.name()))
+				{
+					if (find(lv->m_child_keys.begin(), lv->m_child_keys.end(), key) != lv->m_child_keys.end())
+					{
+						replaceable = false;
+						break;
+					}
+				}
+
+				if (replaceable)
+					replaceableKeys.insert(key);
+			}
+
+			if (replaceableKeys.empty())
+				throw std::runtime_error("Cannot repair category " + cat.name() + " since it contains duplicate keys that cannot be replaced");
+
+			for (auto key : replaceableKeys)
+			{
+				for (auto row : cat)
+					row.assign(key, cat.get_unique_value(key), false, false);
+			}
+
+			continue;
+		}
+
+		break;
+	}
+}
+
 bool reconstruct_pdbx(file &file)
 {
 	if (file.empty())
@@ -1337,7 +1434,7 @@ bool reconstruct_pdbx(file &file, const validator &validator)
 				              iv->m_type != nullptr and
 				              iv->m_type->m_primitive_type == cif::DDL_PrimitiveType::Numb;
 
-				for (std::size_t ix = 0; auto row : cat)
+				for (int ix = 0; auto row : cat)
 				{
 					if (number)
 						row.assign(key, std::to_string(++ix), false, false);
@@ -1408,95 +1505,7 @@ bool reconstruct_pdbx(file &file, const validator &validator)
 				}
 			}
 
-			enum class State
-			{
-				Start,
-				MissingKeys,
-				DuplicateKeys
-			} state = State::Start;
-
-			for (;;)
-			{
-				// See if we can build an index
-				try
-				{
-					cat.set_validator(&validator, db);
-				}
-				catch (const missing_key_error &ex)
-				{
-					if (state == State::MissingKeys)
-					{
-						if (cif::VERBOSE > 0)
-							std::clog << "Repairing failed for category " << cat.name() << ", missing keys remain: " << ex.what() << '\n';
-
-						throw;
-					}
-
-					state = State::MissingKeys;
-
-					auto key = ex.get_key();
-
-					if (cif::VERBOSE > 0)
-						std::clog << "Need to add key " << key << " to category " << cat.name() << '\n';
-
-					for (auto row : cat)
-					{
-						auto ord = row.get<std::string>(key.c_str());
-						if (ord.empty())
-							row.assign({ //
-								{ key, cat.get_unique_value(key) } });
-					}
-
-					continue;
-				}
-				catch (const duplicate_key_error &ex)
-				{
-					if (state == State::DuplicateKeys)
-					{
-						if (cif::VERBOSE > 0)
-							std::clog << "Repairing failed for category " << cat.name() << ", duplicate keys remain: " << ex.what() << '\n';
-
-						throw;
-					}
-
-					state = State::DuplicateKeys;
-
-					if (cif::VERBOSE > 0)
-						std::clog << "Attempt to fix " << cat.name() << " failed: " << ex.what() << '\n';
-
-					// replace items that do not define a relation to a parent
-
-					std::set<std::string> replaceableKeys;
-					for (auto key : cv->m_keys)
-					{
-						bool replaceable = true;
-						for (auto lv : validator.get_links_for_child(cat.name()))
-						{
-							if (find(lv->m_child_keys.begin(), lv->m_child_keys.end(), key) != lv->m_child_keys.end())
-							{
-								replaceable = false;
-								break;
-							}
-						}
-
-						if (replaceable)
-							replaceableKeys.insert(key);
-					}
-
-					if (replaceableKeys.empty())
-						throw std::runtime_error("Cannot repair category " + cat.name() + " since it contains duplicate keys that cannot be replaced");
-
-					for (auto key : replaceableKeys)
-					{
-						for (auto row : cat)
-							row.assign(key, cat.get_unique_value(key), false, false);
-					}
-
-					continue;
-				}
-
-				break;
-			}
+			reconstruct_index_for_category(validator, cat, db);
 		}
 		catch (const std::exception &ex)
 		{
@@ -1537,7 +1546,7 @@ bool reconstruct_pdbx(file &file, const validator &validator)
 
 	if (auto cat = db.get("ndb_poly_seq_scheme"); cat == nullptr or cat->empty())
 		comparePolySeqSchemes(db);
-	
+
 	createPdbxNonpolyScheme(db);
 
 	// Create a minimal set of branch records
@@ -1554,4 +1563,111 @@ bool reconstruct_pdbx(file &file, const validator &validator)
 	return valid and is_valid_pdbx_file(file, validator);
 }
 
+// --------------------------------------------------------------------
+
+void fixup_pdbx(file &file)
+{
+	if (file.empty())
+		throw std::runtime_error("Cannot reconstruct PDBx, file seems to be empty");
+
+	auto &db = file.front();
+
+	if (auto ac = db.get("audit_conform"); ac != nullptr)
+		fixup_pdbx(file, validator_factory::instance().get(*ac));
+	else
+		fixup_pdbx(file, validator_factory::instance().get("mmcif_pdbx.dic"));
+}
+
+void fixup_pdbx(file &file, const validator &validator)
+{
+	if (file.empty())
+		throw std::runtime_error("Cannot reconstruct PDBx, file seems to be empty");
+
+	// assuming the first datablock contains the entry ...
+	auto &db = file.front();
+
+	if (auto cat = db.get("atom_site"); cat == nullptr or cat->empty())
+		throw std::runtime_error("Cannot reconstruct PDBx file, atom data missing");
+
+	// ... and any additional datablock will contain compound information
+	cif::compound_source cs(file);
+
+	// Be silent about missing compound info in fixup
+	auto &cf = compound_factory::instance();
+	bool save_report = cf.get_report_missing();
+	cf.set_report_missing(cif::VERBOSE > 1);
+
+	std::string entry_id;
+
+	// Phenix files do not have an entry record
+	if (auto cat = db.get("entry"); cat == nullptr or cat->empty())
+	{
+		entry_id = db.name();
+		category entry("entry");
+		entry.emplace({ { "id", entry_id } });
+		db.emplace_back(std::move(entry));
+	}
+	else
+	{
+		auto &entry = db["entry"];
+		if (entry.size() != 1)
+			throw std::runtime_error("Unexpected size of entry category");
+
+		entry_id = entry.front().get<std::string>("id");
+	}
+
+	// Start with chem_comp, it is often missing many fields
+	// that can easily be filled in.
+	checkChemCompRecords(db);
+
+	// If the data is really horrible, it might not contain entities
+	if (not db["atom_site"].find_first(key("label_entity_id") != null))
+		createEntityIDs(db);
+
+	// Now see if atom records make sense at all, but in a silent way, this time
+	checkAtomRecords(db);
+
+	db["chem_comp"].reorder_by_index();
+
+	// See if we can easily reconstruct missing data fields in order to create an index
+	for (auto &cat : db)
+	{
+		try
+		{
+			cat.set_validator(&validator, db);
+		}
+		catch (const missing_key_error &)
+		{
+			reconstruct_index_for_category(validator, cat, db);
+		}
+	}
+
+	db.set_validator(&validator);
+
+	// Now create any missing categories
+	// Next make sure we have struct_asym records
+	if (auto cat = db.get("struct_asym"); cat == nullptr or cat->empty())
+		createStructAsym(db);
+
+	if (auto cat = db.get("entity"); cat == nullptr or cat->empty())
+		createEntity(db);
+
+	if (auto cat = db.get("pdbx_poly_seq_scheme"); cat == nullptr or cat->empty())
+		createPdbxPolySeqScheme(db);
+
+	if (auto cat = db.get("ndb_poly_seq_scheme"); cat == nullptr or cat->empty())
+		comparePolySeqSchemes(db);
+
+	createPdbxNonpolyScheme(db);
+
+	// Create a minimal set of branch records
+	createPdbxBranchScheme(db);
+
+	// fill in missing formula_weight, e.g.
+	checkEntities(db);
+
+	// That's it
+	cf.set_report_missing(save_report);
+}
+
 } // namespace cif::pdb
